<?php

namespace Drupal\seostudio\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\seostudio\NodeDataFetcher;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;

class SeoStudioController extends ControllerBase {

  protected $nodeDataFetcher;
  protected $themeManager;

  public function __construct(NodeDataFetcher $nodeDataFetcher, ThemeManagerInterface $themeManager) {
    $this->nodeDataFetcher = $nodeDataFetcher;
    $this->themeManager = $themeManager;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('seostudio.node_data_fetcher'),
      $container->get('theme.manager')
    );
  }

  public function content(NodeInterface $node) {
    $metaData = $this->nodeDataFetcher->fetchMetatagData($node, TRUE, $node->language());

    $module_path = '/modules/contrib/seostudio';
    // Use the extension.path.resolver service to get the module path
    if (\Drupal::hasService('extension.path.resolver')) {
      $module_path = \Drupal::service('extension.path.resolver')->getPath('module', 'seostudio');
    } else {
      // fallback for D8
      $module_path = drupal_get_path('module', 'seostudio');
    }
    // show a fieldset on top (closed by default)
    // to show the current metatags in a <code><pre> block
    // loop over the metatags and show them in a <code><pre> block
    $metaDataCodeblock = '<code>';
    foreach ($metaData as $key => $value) {
      if(!empty($value)) {
        $metaDataCodeblock .= '<pre><strong>' . $key . '</strong>: ' . $value . '</pre>';
      }
    }
    $metaDataCodeblock .= '</code>';

    $current_metatags = [
      '#type' => 'details',
      '#title' => $this->t('Raw metatags'),
      '#open' => FALSE,
      '#attributes' => ['id' => 'seo-studio-current-metatags'],
      'code' => [
        '#type' => 'html_tag',
        '#tag' => 'code',
        '#value' => $metaDataCodeblock,
      ],
    ];

    $left_column = [
      '#theme' => 'seo_studio',
      '#title' => !$this->isToken($metaData['title'] ?? '') && !empty($metaData['title']) ? ($metaData['title'] ?? $this->t('No title set yet')) : $this->t('No title set yet'),
      '#og_title' => !$this->isToken($metaData['og_title'] ?? '') && !empty($metaData['og_title']) ? ($metaData['og_title'] ?? $this->t('No open graph title set yet')) : $this->t('No open graph title set yet'),
      '#description' => !$this->isToken($metaData['description'] ?? '') && !empty($metaData['description']) ? ($metaData['description'] ?? $this->t('No description set yet')) : $this->t('No description set yet'),
      '#image_url' => !$this->isToken($metaData['og_image_url'] ?? '') && !empty($metaData['og_image_url']) ? ($metaData['og_image_url'] ?? '/' . $module_path . '/images/meta-tags.jpg') : '/' . $module_path . '/images/meta-tags.jpg',
      '#twitter_card_url' => !$this->isToken($metaData['twitter_cards_image'] ?? '') && !empty($metaData['twitter_cards_image']) ? ($metaData['twitter_cards_image'] ?? '/' . $module_path . '/images/meta-tags.jpg') : '/' . $module_path . '/images/meta-tags.jpg',
      '#link' => isset($metaData['canonical_url']) && !$this->isToken($metaData['canonical_url']) && !empty($metaData['canonical_url']) ? $metaData['canonical_url'] : 'no canonical url metatag found.',
      '#attributes' => ['id' => 'seo-studio-left-content'],
    ];

    $right_column = \Drupal::formBuilder()->getForm('\Drupal\seostudio\Form\SeoStudioForm', $node);

    $default_classes = [
      'layout-node-form',
      'clearfix',
      'layout-form',
    ];

    $gin_classes = [
      'clearfix',
      'gin-layer-wrapper'
    ];

    $left_column_classes = [
      'layout__region',
      'layout-region',
      'layout-region--node-main',
      'layout-region--main'
    ];

    $right_column_classes = [
      'layout__region',
      'layout-region',
      'layout-region--node-secondary',
      'layout-region--secondary'
    ];

    $active_theme = $this->themeManager->getActiveTheme()->getName();
    $theme_specific_classes = ($active_theme === 'gin') ? $gin_classes : [];

    $build = [
      '#type' => 'container',
      '#attributes' => ['class' => array_merge($default_classes, $theme_specific_classes)],
      'heading' => [
        '#type' => 'container',
        'content' => $current_metatags
      ],
      'right_column' => [
        '#type' => 'container',
        '#attributes' => ['class' => \Drupal::moduleHandler()->moduleExists('flaregpt') ? array_merge($theme_specific_classes, $right_column_classes) : ''],
        'content' => $right_column,
      ],
      'left_column' => [
        '#type' => 'container',
        '#attributes' => ['class' => array_merge($theme_specific_classes, $left_column_classes)],
        'content' => $left_column,
      ],
    ];


    if ($this->themeManager->getActiveTheme()->getName() === 'claro') {
      $build['#attached']['library'][] = 'claro/node-form';
      $build['#attached']['library'][] = 'claro/form-two-columns';
    }

    $build['#attached']['library'][] = 'core/drupal.ajax';
    $build['#attached']['library'][] = 'seostudio/seostudio';

    return $build;
  }

  public function access(NodeInterface $node) {
    // Retrieve the configuration for seostudio settings.
    $config = $this->config('seostudio.settings');
    // Get the specific configuration for the node type.
    $enabled = $config->get($node->getType())['enabled'];
    // Check if the 'enabled' value is 1 and if the user has the 'administer meta tags' permission.
    if ($enabled === 1 && \Drupal::currentUser()->hasPermission('administer meta tags')) {
      return AccessResult::allowed()->addCacheTags(['config:seostudio.settings.' . $node->getType()]);
    }
    // If the conditions are not met, access is forbidden.
    return AccessResult::forbidden();
  }

  /**
   * Check if a value is a token.
   *
   * @param string $value
   *   The value to check.
   *
   * @return bool
   *   TRUE if the value is a token, FALSE otherwise.
   */
  private function isToken($value) {
    return preg_match('/\[[a-zA-Z_:]+\]/', $value);
  }

  /**
   * Get title.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   *
   * @return string
   *   The page title.
   */
  public function getTitle(RouteMatchInterface $route_match) {
    // Get the current node from the route match.
    $node = $route_match->getParameter('node');

    // Ensure the parameter is a node object.
    if ($node instanceof NodeInterface) {
      $pageTitle = $node->getTitle();
    } else {
      // Fallback if the node is not found or not a valid node.
      $pageTitle = $this->t('Unknown Page');
    }

    return $this->t('Metatags for @pagetitle', ['@pagetitle' => $pageTitle]);
  }

}
