<?php

namespace Drupal\seostudio\Form;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\seostudio\NodeDataFetcher;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\Core\Messenger\MessengerInterface;

class SeoStudioForm extends FormBase {

  /**
   * The temporary storage factory.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStoreFactory
   */
  protected $tempStoreFactory;


  public function __construct(
    NodeDataFetcher $nodeDataFetcher,
    PrivateTempStoreFactory $temp_store_factory,
    MessengerInterface $messenger) {
    $this->nodeDataFetcher = $nodeDataFetcher;
    $this->tempStoreFactory = $temp_store_factory;
    $this->messenger = $messenger;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('seostudio.node_data_fetcher'),
      $container->get('tempstore.private'),
      $container->get('messenger')
    );
  }

  public function getFormId() {
    return 'seo_studio_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state, NodeInterface $node = NULL) {

    // Check if the 'flaregpt' module is installed
    if (!\Drupal::moduleHandler()->moduleExists('flaregpt')) {
      return $form;
    }

    // If the module is installed, continue with the rest of the form building

    $config = \Drupal::config('flaregpt.settings');
    $seo_studio_config = \Drupal::config('seostudio.settings');
    $images_enabled = $seo_studio_config->get($node->getType())['meta_image']['meta_images'];

    $jsonld_enabled = $seo_studio_config->get($node->getType())['schema_org']['jsonld'];

    $metatagFieldName = $this->nodeDataFetcher->findMetatagField($node);

    $api_key = $config->get('api_key');

    $form['select'] = [
      '#type' => 'select',
      '#title' => $this->t('Select an option'),
      '#options' => [
        'metatag' => $this->t('Generate title and description'),
      ],
      '#disabled' => isset($api_key) ? FALSE : TRUE
    ];

    if($images_enabled) {
        $form['select']['#options']['metaimage'] = $this->t('Generate meta image');
    }

    if($jsonld_enabled) {
        $form['select']['#options']['jsonld'] = $this->t('Generate JSON-LD snippet');
    }

    $form['ajax_submit'] = [
      '#type' => 'button',  // Use 'button' instead of 'submit' to avoid triggering form submission
      '#value' => $this->t('Generate'),
      '#ajax' => [
        'callback' => '::ajaxSubmitHandler',
        'wrapper' => 'seo-studio-left-content',
        'event' => 'click',
      ],
      '#disabled' => isset($api_key) ? FALSE : TRUE
    ];

    $form['save_to_node'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save to node'),
      '#disabled' => isset($api_key) ? FALSE : TRUE
    ];

    if(!$api_key) {
      $form['api_key_warning'] = [
        '#type' => 'markup',
        '#markup' => $this->t('<div class="messages messages--warning">No API Key is set. Please provide a working API Key from <a href=":url" target="_blank">FlareGPT.ai</a> to enable functionality.</div>', [':url' => 'https://flaregpt.ai']),
      ];
    }

    if(!$metatagFieldName) {
      $form['metatag_warning'] = [
        '#type' => 'markup',
        '#markup' => $this->t('<div class="messages messages--warning">No metatag field found. Please add a metatag field to the content type to enable functionality.</div>'),
      ];
      // disable the form
      $form['select']['#disabled'] = TRUE;
      $form['ajax_submit']['#disabled'] = TRUE;
      $form['save_to_node']['#disabled'] = TRUE;
    }

    $form['#prefix'] = '<div id="seo-studio-form">';
    $form['#suffix'] = '</div>';

    $form['#attached']['library'][] = 'core/drupal.ajax';

    return $form;
  }

  public function ajaxSubmitHandler(array &$form, FormStateInterface $form_state) {
    $selected_option = $form_state->getValue('select');
    $node = \Drupal::routeMatch()->getParameter('node');
    $metaData = $this->nodeDataFetcher->fetchMetatagData($node, TRUE, $node->language());

    $ajax_response = new AjaxResponse();

    if ($selected_option && $node instanceof NodeInterface) {
      try {
        // get current language
        $language =  \Drupal::languageManager()->getCurrentLanguage();
        $response = \Drupal::service('flaregpt.metatag_node_processor')->processNode($node, $selected_option, $language);
        if (isset($response['error'])) {
          // Handle the error response
          $message = $response['error'];
          $this->messenger->addError($message);

          // Render messages and capture output
          $messages = ['#type' => 'status_messages'];
          $rendered_messages = \Drupal::service('renderer')->renderRoot($messages);

          $ajax_response->addCommand(new HtmlCommand('#ajax-message', $rendered_messages));

        }
        else {
          $module_path = '/modules/contrib/seostudio';
          if (\Drupal::hasService('extension.path.resolver')) {
            $module_path = \Drupal::service('extension.path.resolver')->getPath('module', 'seostudio');
          } else {
            // fallback for D8
            $module_path = drupal_get_path('module', 'seostudio');
          }
          $image_path = '/' . $module_path . '/images/meta-tags.jpg';

          $content = [
            '#theme' => 'seo_studio',
            '#title' => !empty($response['title']) && !$this->isToken($response['title']) ? $response['title'] : (!empty($metaData['title']) && !$this->isToken($metaData['title']) ? $metaData['title'] : t('No title set yet')),
            '#description' => !empty($response['description']) && !$this->isToken($response['description']) ? $response['description'] : (!empty($metaData['description']) && !$this->isToken($metaData['description']) ? $metaData['description'] : t('No description set yet')),
            '#image_url' => !empty($response['meta_image_url']) && !$this->isToken($response['meta_image_url']) ? $response['meta_image_url'] : (!empty($metaData['og_image_url']) && !$this->isToken($metaData['og_image_url']) ? $metaData['og_image_url'] : $image_path),
            '#twitter_card_url' => !empty($response['twitter_card_url']) && !$this->isToken($response['twitter_card_url']) ? $response['twitter_card_url'] : (!empty($metaData['twitter_cards_image']) && !$this->isToken($metaData['twitter_cards_image']) ? $metaData['twitter_cards_image'] : $image_path),
            '#link' => !empty($metaData['canonical_url']) && !$this->isToken($metaData['canonical_url']) ? $metaData['canonical_url'] : 'no canonical url metatag found.',
            '#jsonld' => !empty($response['jsonld']) ? $response['jsonld'] : NULL,
          ];

          $temp_store = $this->tempStoreFactory->get('seostudio');
          $temp_store->set('metatag_update_data_' . $node->id(), $response);
          $rendered_content = \Drupal::service('renderer')->render($content);

          $ajax_response->addCommand(new HtmlCommand('#ajax-message', ''));

          if ($response) {
            $form['save_to_node']['#access'] = TRUE;
            $form['submit']['#value'] = $this->t('Resubmit');
          }
          $ajax_response->addCommand(new ReplaceCommand('#seo-studio-left-content', $rendered_content));

        }
      }
      catch (\Exception $e) {
        \Drupal::logger('seostudio')->error($e->getMessage());
        return ['error' => t('Failed to process Seo Studio form for node @node_id. Error: @error', [
          '@node_id' => $node->id(),
          '@error' => $e->getMessage()
        ])];
      }
    }
    else {
      $message = $this->t('Invalid selection or node.');
      \Drupal::messenger()->addError($message['message']);
      $ajax_response->addCommand(new HtmlCommand('#ajax-message', $message['message']));
    }

    $form_state->setRebuild();
    $ajax_response->addCommand(new ReplaceCommand('#seo-studio-form', $form));

    return $ajax_response;
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {

    $node = \Drupal::routeMatch()->getParameter('node');

    $temp_store = $this->tempStoreFactory->get('seostudio');
    $metatags =  $temp_store->get('metatag_update_data_' . $node->id(), NULL);

    if(!$metatags) {
      \Drupal::messenger()->addError($this->t('Failed to update metatags for node @node_id. Did you click \'Generate metatags\' first?', ['@node_id' => $node->id()]));
      $form_state->setRedirect('seostudio.seostudio', ['node' => $node->id()]);
      return;
    }

    // Service call to save metatags
    $metatagNodeSaverService = \Drupal::service('flaregpt.metatag_node_saver_service');

    // Fetch existing tags, but keep tokens
    $existingTags = $this->nodeDataFetcher->fetchMetatagData($node, FALSE, $node->language());

    $saveResult = $metatagNodeSaverService->saveMetatags($node, $metatags, $node->language()->getId(), $existingTags);

    $temp_store->delete('metatag_update_data_' . $node->id());

    if ($saveResult) {
      \Drupal::messenger()->addMessage($this->t('Metatags updated for \'@node_label\'.', ['@node_label' => $node->label()]));
    } else {
      \Drupal::messenger()->addError($this->t('Failed to update metatags.', ['@node_id' => $node->id()]));
    }

      // Redirect the user back to the node page
      $form_state->setRedirect('seostudio.seostudio', ['node' => $node->id()]);
    }

  /**
   * Check if a value is a token.
   *
   * @param string $value
   *   The value to check.
   *
   * @return bool
   *   TRUE if the value is a token, FALSE otherwise.
   */
  private function isToken($value) {
    return preg_match('/\[[a-zA-Z_:]+\]/', $value);
  }

}



