<?php

namespace Drupal\seostudio;

use Drupal\node\NodeInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Utility\Token;

class NodeDataFetcher {

  protected $renderer;
  protected $entityTypeManager;
  protected $languageManager;
  protected $moduleHandler;
  protected $token;

  public function __construct(RendererInterface $renderer, EntityTypeManagerInterface $entityTypeManager, LanguageManagerInterface $languageManager, ModuleHandlerInterface $moduleHandler, Token $token) {
    $this->renderer = $renderer;
    $this->entityTypeManager = $entityTypeManager;
    $this->languageManager = $languageManager;
    $this->moduleHandler = $moduleHandler;
    $this->token = $token;
  }

  public function fetchNodeData(NodeInterface $node) {
    $title = $node->getTitle();

    if ($node->hasField('body') && !$node->get('body')->isEmpty()) {
      $body = $node->get('body')->value;
      $teaser = substr($body, 0, 500);
    } else {
      $viewBuilder = $this->entityTypeManager->getViewBuilder('node');
      $viewBuilderRendered = $viewBuilder->view($node, 'full');
      $renderedNode = $this->renderer->renderRoot($viewBuilderRendered);
      $strippedContent = strip_tags($renderedNode);
      $teaser = substr($strippedContent, 0, 1500);
    }

    return [
      'title' => $title,
      'teaser' => $teaser,
    ];
  }

  public function findMetatagField(NodeInterface $node) {
    $fields = $node->getFieldDefinitions();
    foreach ($fields as $field_name => $field_definition) {
      if ($field_definition->getType() === 'metatag') {
        return $field_name;
      }
    }
    return null;
  }

  public function fetchMetatagData(NodeInterface $node, $transformTokens = TRUE, $langCode = NULL) {
    // if langcode type of Drupal\Core\Language\Language
    if ($langCode instanceof Drupal\Core\Language\LanguageInterface) {
      $node = $node->getTranslation($langCode->getId());
    }

    $metatagFieldName = $this->findMetatagField($node);

    if ($metatagFieldName === null) {
      return false; // No metatag field found
    }

    $metatags = \Drupal::service('metatag.manager')->tagsFromEntityWithDefaults($node);

    if($transformTokens) {
      $metatags = $this->replaceTokens($node, $metatags);
    }

    return $metatags;
  }

  private function replaceTokens($node, $metatags) {
    // Load the token service
    $token_service = \Drupal::token();
    $site_base_url = \Drupal::request()->getSchemeAndHttpHost();
    $base_path = \Drupal::request()->getBasePath();

    // Loop through metatags and replace tokens
    foreach ($metatags as $key => &$tag) {
      // Include URL context for token replacement
      $context = [
        'node' => $node,
        'url' => $node->toUrl('canonical', ['absolute' => TRUE])->toString(),
        'site' => ['base-url' => $site_base_url], // Explicitly add base URL context
      ];

      // Replace tokens with the appropriate context
      $options = ['clear' => TRUE];
      $tag = $token_service->replace($tag, $context, $options);

      // Ensure that the description is stripped of HTML tags
      if ($key === 'description' || $key === 'twitter_cards_description') {
        $tag = strip_tags($tag);
      }

      // Prepend the base URL to image paths if they are relative
      if (in_array($key, ['image_src', 'meta_image_url', 'og_image', 'og_image_secure_url', 'og_image_url', 'twitter_card_url', 'twitter_cards_image']) && !parse_url($tag, PHP_URL_HOST)) {
        // Use file_url_transform_relative to handle relative paths correctly
        if (!empty($tag)) {
          if (\Drupal::hasService('file_url_generator')) {
            $tag = \Drupal::service('file_url_generator')->transformRelative($tag);
          } else {
            // Fallback if the service does not exist
            if (function_exists('file_url_transform_relative')) {
              $tag = file_url_transform_relative($tag);
            }
          }
          $tag = $site_base_url . $base_path . $tag;
        }
      }
    }

    return $metatags;
  }

  /**
   * Checks if a given string contains a token pattern.
   *
   * @param string $string
   *   The string to check.
   *
   * @return bool
   *   TRUE if the string contains a token pattern, FALSE otherwise.
   */
  function contains_token_pattern($string) {
    // Regular expression to match token patterns like [type:key].
    $pattern = '/\[[a-zA-Z0-9_]+:[a-zA-Z0-9_]+\]/';

    // Check if the string matches the pattern.
    return preg_match($pattern, $string) === 1;
  }

}
